# -*- coding: utf-8 -*-
from .BaseTestCase import BaseDXTestCase
from plone.namedfile.file import NamedFile
from Products.CMFDiffTool import namedfile
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.interfaces import IDifference


class BinaryDiffTestCase(BaseDXTestCase):

    def test_should_detect_different_filename(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            file=NamedFile(data='contents', filename=u'blah.txt')
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            file=NamedFile(data='contents', filename=u'bleh.txt')
        )
        obj2 = self.portal['obj2']

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, 'file')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)

    def test_should_detect_different_data(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            file=NamedFile(data='contents', filename=u'f.txt')
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            file=NamedFile(data='different contents', filename=u'f.txt')
        )
        obj2 = self.portal['obj2']

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, 'file')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertFalse(diff.same)

    def test_should_detect_same_data_and_filename(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            file=NamedFile(data='contents', filename=u'f.txt')
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            file=NamedFile(data='contents', filename=u'f.txt')
        )
        obj2 = self.portal['obj2']

        diff = namedfile.NamedFileBinaryDiff(obj1, obj2, 'file')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertTrue(diff.same)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing.bbb import PTC_FIXTURE
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from zope.component import getSiteManager
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


TEST_CONTENT_TYPE_ID = 'TestContentType'

VOCABULARY_TUPLES = [
    (u'first_value', u'First Title'),
    (u'second_value', None),
]

VOCABULARY = SimpleVocabulary(
    [SimpleTerm(value=v, title=t) for (v, t) in VOCABULARY_TUPLES])


def vocabulary_factory(context):
    return VOCABULARY


class DXLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE, )

    def setUpPloneSite(self, portal):
        '''Set up additional products and ZCML required to test
        this product.
        '''
        # setup dexterity
        types_tool = getToolByName(portal, 'portal_types')

        sm = getSiteManager(portal)
        sm.registerUtility(
            component=vocabulary_factory,
            provided=IVocabularyFactory,
            name=u'Products.CMFDiffTool.testing.VOCABULARY'
        )

        fti = DexterityFTI(
            TEST_CONTENT_TYPE_ID,
            factory=TEST_CONTENT_TYPE_ID,
            global_allow=True,
            behaviors=(
                'plone.app.versioningbehavior.behaviors.IVersionable',
                'plone.app.dexterity.behaviors.metadata.IBasic',
                'plone.app.dexterity.behaviors.metadata.IRelatedItems',
                'plone.app.contenttypes.behaviors.collection.ICollection',
            ),
            model_source='''
            <model xmlns='http://namespaces.plone.org/supermodel/schema'>
                <schema>
                    <field name='text' type='zope.schema.Text'>
                        <title>Text</title>
                        <required>False</required>
                    </field>
                    <field name='file' type='plone.namedfile.field.NamedFile'>
                        <title>File</title>
                        <required>False</required>
                    </field>
                    <field name='date' type='zope.schema.Date'>
                        <title>Date</title>
                        <required>False</required>
                    </field>
                    <field name='files' type='zope.schema.List'>
                        <title>Date</title>
                        <required>False</required>
                        <value_type type='plone.namedfile.field.NamedFile'>
                            <title>Val</title>
                        </value_type>
                    </field>
                    <field name='choice' type='zope.schema.Choice'>
                        <title>Choice</title>
                        <required>False</required>
                        <vocabulary>Products.CMFDiffTool.testing.VOCABULARY</vocabulary>
                    </field>
                </schema>
            </model>
            '''
        )
        types_tool._setObject(TEST_CONTENT_TYPE_ID, fti)

PACKAGE_DX_FIXTURE = DXLayer()


class ATLayer(PloneSandboxLayer):

    defaultBases = (PTC_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # setup schema extender if available
        try:
            from archetypes import schemaextender
            self.loadZCML(package=schemaextender)
        except ImportError:
            pass

PACKAGE_AT_FIXTURE = ATLayer()

CMFDiffToolLayer = FunctionalTesting(
    bases=(PTC_FIXTURE, ), name='Products.CMFDiffTool:functional')
CMFDiffToolDXLayer = FunctionalTesting(
    bases=(PACKAGE_DX_FIXTURE, ), name='Products.CMFDiffTool.DX:functional')
CMFDiffToolATLayer = FunctionalTesting(
    bases=(PACKAGE_AT_FIXTURE, ), name='Products.CMFDiffTool.AT:functional')



# -*- coding: utf-8 -*-
# Interface definitions
from zope.interface import Interface, Attribute


class IDiffTool(Interface):
    """An interface to compute object differences via pluggable
       difference engine"""

    id = Attribute('id', 'Must be set to "portal_diff"')

    def listDiffTypes():
        """List the names of the available difference types"""

    def setDiffForPortalType(pt_name, mapping):
        """Set the difference type(s) for the specific portal type

        mapping is a dictionary where each key is an attribute or
        method on the given portal type, and the value is the name of
        a difference type."""

    def getDiffForPortalType(pt_name):
        """Returns a dictionary where each key is an attribute or
        method on the given portal type, and the value is the name of
        a difference type."""

    def computeDiff(ob1, ob2):
        """Compute the differences from ob1 to ob2 (ie. ob2 - ob1).

        The result will be a list of objects that implements the
        IDifference interface and represent the differences between
        ob1 and ob2."""

    def createChangeSet(ob1, ob2):
        """Returns a ChangeSet object that represents the differences
        between ob1 and ob2 (ie. ob2 - ob1) ."""


class IDifference(Interface):
    """An interface for interacting with the difference between two
    objects"""

    meta_type = Attribute('title', 'A human readable name for the diff type')
    field = Attribute('field', 'The name of the field being compared')
    same = Attribute('same',
                     'True if the fields are the "same" '
                     '(whatever that means for this difference)')
    oldValue = Attribute('oldValue', 'The old field value being compared')
    newValue = Attribute('newValue', 'The new field value being compared')
    oldFilename = Attribute('oldFilename',
                            'The old filename for the field being compared')
    newFilename = Attribute('newFilename',
                            'The new filename for the field being compared')

    def testChanges(ob):
        """Test the specified object to determine if the change set will apply cleanly.

        Returns None if there would be no erros
        """

    def applyChanges(ob):
        """Update the specified object with the difference"""

    def filenameTitle(self, filename):
        """Translate the filename leading text
        """


class IStringDifference(IDifference):
    """An anterface for interacting with the difference between two
    string (text) objects"""

    def getLineDiffs():
        """Return a list of differences between the two objects on a
        line-by-line basis

        Each difference is a 5-tuple as described here:
        http://www.python.org/doc/2.1.3/lib/sequence-matcher.html#l2h-721

        The interpretation of these tuples depends on the difference class"""


class IChangeSet(Interface):
    """And interface representing all of the differences between two objects"""

    same = Attribute('same', 'True if the fields are the "same"')

    def computeDiff(ob1, ob2, recursive=1, exclude=None):
        """Compute the differences from ob1 to ob2 (ie. ob2 - ob1).

        If resursive is 1, compute differences between subobjects of
        ob1 and ob2 as well, excluding any subobjects whose IDs are
        listed in exclude

        The results can be accessed through getDiffs()"""

    def testChanges(ob):
        """Test the specified object to determine if the change set will apply cleanly.

        Returns None if there would be no erros
        """

    def applyChanges(ob):
        """Apply the computed changes to the specified object"""

    def getDiffs():
        """Returns the list of differences between the two objects.

        Each difference is a single object implementing the
        IDifference interface"""

    def getSubDiffs():
        """If the ChangeSet was computed recursively, returns a list
           of ChangeSet objects representing subobject differences

           Each ChangeSet will have the same ID as the objects whose
           difference it represents.
           """

    def getAddedItems():
        """If the ChangeSet was computed recursively, returns the list
        of IDs of items that were added.

        A copy of these items is available as a cubject of the ChangeSet
        """

    def getRemovedItems():
        """If the ChangeSet was computed recursively, returns the list
        of IDs of items that were removed"""



# -*- coding: utf-8 -*-
def safe_unicode(value):
    if isinstance(value, unicode):
        return value
    try:
        value = unicode(value)
    except UnicodeDecodeError:
        value = value.decode('utf-8', 'replace')
    return value


def safe_utf8(value):
    return safe_unicode(value).encode('utf-8')



# -*- coding: utf-8 -*-
# Copyright (c) 2003 The Connexions Project, All Rights Reserved
# Written by Brent Hendricks

from Products.CMFDiffTool.interfaces import IDifference  # NOQA
from Products.CMFDiffTool.interfaces import IDiffTool as portal_diff  # NOQA



# -*- coding: utf-8 -*-
from .BaseTestCase import BaseDXTestCase
from datetime import date
from plone.namedfile import NamedFile
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.dexteritydiff import DexterityCompoundDiff
from Products.CMFDiffTool.dexteritydiff import EXCLUDED_FIELDS
from Products.CMFDiffTool.interfaces import IDifference


class DexterityDiffTestCase(BaseDXTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()

    def test_should_diff(self):
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            title=u'Object 1',
            description=u'Desc 1',
            text=u'Text 1'
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            title=u'Object 2',
            text=u'Text 2'
        )
        obj2 = self.portal['obj2']

        diffs = DexterityCompoundDiff(obj1, obj2, 'any')
        for d in diffs:
            self.assertTrue(IDifference.providedBy(d))
            self.assertFalse(d.field in EXCLUDED_FIELDS)
            if d.field in ['title', 'description', 'text']:
                self.assertFalse(
                    d.same, 'Field %s should be different.' % d.field)
            else:
                self.assertTrue(d.same, 'Field %s should be equal.' % d.field)

    def test_should_provide_inline_diff_for_date_field(self):
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            date=date(2001, 1, 1),
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            date=date(2001, 1, 2),
        )
        obj2 = self.portal['obj2']

        diffs = DexterityCompoundDiff(obj1, obj2, 'any')
        for d in diffs:
            if d.field == 'date':
                self.assertTrue(d.inline_diff())

    def test_should_provide_inline_diff_for_file_list_field(self):
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            files=None,
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            files=[NamedFile(data='data', filename=u'a.txt')],
        )
        obj2 = self.portal['obj2']

        diffs = DexterityCompoundDiff(obj1, obj2, 'any')
        for d in diffs:
            if d.field == 'files':
                inline_diff = d.inline_diff()
                self.assertTrue(inline_diff)
                self.assertTrue(obj2.files[0].filename in inline_diff)

    def test_should_provide_diff_for_behaviors_fields(self):
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
            date=date(2001, 1, 1),
        )
        obj1 = self.portal['obj1']

        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
            date=date(2001, 1, 2),
        )
        obj2 = self.portal['obj2']

        diffs = DexterityCompoundDiff(obj1, obj2, 'any')
        fields = [d.field for d in diffs]
        self.assertIn('title', fields)
        self.assertIn('description', fields)



# -*- coding: utf-8 -*-
"""CMFDiffTool.py

   Calculate differences between content objects
"""

from Acquisition import aq_base
from App.class_init import InitializeClass
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFDiffTool import CMFDiffToolMessageFactory as _
from Products.CMFDiffTool.interfaces import IDifference
from Products.CMFPlone.utils import safe_hasattr
from zope.i18n import translate
from zope.interface import implements

import Acquisition


class BaseDiff:
    """Basic diff type"""

    implements(IDifference)
    __allow_access_to_unprotected_subobjects__ = 1
    meta_type = 'Base Diff'

    def __init__(self, obj1, obj2, field, id1=None, id2=None,
                 field_name=None, field_label=None, schemata=None):
        self.field = field
        self.oldValue = _getValue(obj1, field, field_name)
        self.newValue = _getValue(obj2, field, field_name)
        self.same = (self.oldValue == self.newValue)
        if not id1 and safe_hasattr(obj1, 'getId'):
            id1 = obj1.getId()
        if not id2 and safe_hasattr(obj2, 'getId'):
            id2 = obj2.getId()
        self.id1 = id1
        self.id2 = id2
        self.label = field_label or field
        self.schemata = schemata or 'default'
        fld1 = _getValue(obj1, field, field_name, convert_to_str=False)
        fld2 = _getValue(obj2, field, field_name, convert_to_str=False)
        if safe_hasattr(fld1, 'getFilename'):
            self.oldFilename = fld1.getFilename()
        else:
            self.oldFilename = None
        if safe_hasattr(fld2, 'getFilename'):
            self.newFilename = fld2.getFilename()
        else:
            self.newFilename = None
        if self.oldFilename is not None and self.newFilename is not None \
           and self.same:
            self.same = (self.oldFilename == self.newFilename)

    def testChanges(self, ob):
        """Test the specified object to determine if the change set
        will apply without errors"""
        pass

    def applyChanges(self, ob):
        """Update the specified object with the difference"""
        pass

    def filenameTitle(self, filename):
        """Translate the filename leading text
        """
        msg = _(u"Filename: ${filename}",
                mapping={'filename': filename})
        return translate(msg)


def _getValue(ob, field, field_name, convert_to_str=True):
    # Check for the attribute without acquisition.  If it's there,
    # grab it *with* acquisition, so things like ComputedAttribute
    # will work
    if IDexterityContent.providedBy(ob) and field:
        value = getattr(ob, field, None)
    elif field and safe_hasattr(aq_base(ob), field):
        value = getattr(ob, field)
    elif safe_hasattr(aq_base(ob), 'getField'):
        # Archetypes with an adapter extended schema needs special handling
        field = ob.getField(field_name)
        if field is None:
            raise AttributeError(field)
        value = field.getAccessor(ob)
    else:
        raise AttributeError(field)

    # Handle case where the field is a method
    try:
        value = value()
    except (AttributeError, TypeError):
        pass

    if convert_to_str:
        # If this is some object, convert it to a string
        try:
            if isinstance(value, Acquisition.Implicit):
                value = str(value)
        except TypeError:
            pass

    return value

InitializeClass(BaseDiff)



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.interfaces import IDiffTool
from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects
from Products.GenericSetup.utils import XMLAdapterBase
from zope.interface import implements


class DiffToolXMLAdapter(XMLAdapterBase):
    '''In- and exporter for DiffTool.
    '''
    implements(IDiffTool)

    name = 'diff_tool'

    def _exportNode(self):
        '''Export the object as a DOM node.
        '''
        node = self._doc.createElement('object')
        node.appendChild(self._extractDiffToolSettings())

        self._logger.info('DiffTool settings exported.')
        return node

    def _importNode(self, node):
        if self.environ.shouldPurge():
            self._purgeDiffToolSettings()

        self._initDiffToolSettings(node)
        self._logger.info('DiffTool settings imported.')

    def _purgeDiffToolSettings(self):
        self.context.manage_editDiffFields({})

    def _initDiffToolSettings(self, node):
        for child in node.childNodes:
            if child.nodeName == 'difftypes':
                for type_entry in child.getElementsByTagName('type'):
                    ptype = type_entry.getAttribute('portal_type')
                    fields = {}
                    for field in type_entry.getElementsByTagName('field'):
                        name = field.getAttribute('name')
                        diff = field.getAttribute('difftype')
                        fields[name] = diff
                        self.context.setDiffForPortalType(ptype, fields)

    def _extractDiffToolSettings(self):
        node = self._doc.createElement('difftypes')
        ttool = getToolByName(self.context, 'portal_types')
        for ptype in ttool.listContentTypes():
            diffs = self.context.getDiffForPortalType(ptype)
            if diffs:
                child = self._doc.createElement('type')
                child.setAttribute('portal_type', ptype)
                node.appendChild(child)
            for field_name, diff in diffs.items():
                field = self._doc.createElement('field')
                field.setAttribute('name', field_name)
                field.setAttribute('difftype', diff)
                child.appendChild(field)
        return node


def importDiffTool(context):
    '''Import Factory Tool configuration.
    '''
    site = context.getSite()
    tool = getToolByName(site, 'portal_diff', None)

    if tool is not None:
        importObjects(tool, '', context)


def exportDiffTool(context):
    '''Export Factory Tool configuration.
    '''
    site = context.getSite()
    tool = getToolByName(site, 'portal_diff', None)
    if tool is None:
        logger = context.getLogger('difftool')
        logger.info('Nothing to export.')
        return

    exportObjects(tool, '', context)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import getAdditionalSchemata
from Products.CMFDiffTool.TextDiff import AsTextDiff
from zope.component import getUtility


def title_or_value(vocabulary, value):
    """
    Given a `vocabulary` and a `value` in that vocabulary, return the
    corresponding title or `value` if there is no title.
    """
    try:
        return vocabulary.getTerm(value).title or value
    except LookupError:
        # can happen in some cases, e.g. when using
        # zope.schema.vocabulary.TreeVocabulary
        # XXX: There might be a better solution than just returning the value
        return value


def get_schemas(obj):
    """Return a tuple (schema, additional_schemata)."""
    fti = getUtility(IDexterityFTI, name=obj.portal_type)
    schema = fti.lookupSchema()
    additional_schemata = getAdditionalSchemata(context=obj)
    return (schema, additional_schemata)


def get_field_object(obj, field_name):
    """
    Return the `zope.schema.Field` object corresponding to `field_name` in
    `obj`. Return `None` if not found.
    """
    (schema, additional_schemata) = get_schemas(obj)
    schemas = [schema] + list(additional_schemata)
    for s in schemas:
        field = s.get(field_name, None)
        if field is not None:
            return field
    return None


class ChoiceDiff(AsTextDiff):
    """
    Diff for choice fields.

    It's implemented as an specialization of `AsTextDiff`. The difference is
    that this class tries to obtain the title corresponding to the value from
    the vocabulary associated with the field in order to provide an
    user-friendlier inline diff to the user.
    """

    def __init__(self, obj1, obj2, field, id1=None, id2=None, field_name=None,
                 field_label=None, schemata=None):
        AsTextDiff.__init__(self, obj1, obj2, field, id1, id2, field_name,
                            field_label, schemata)
        self._vocabulary = None

        # Tries to find a vocabulary. First we need to find an object and
        # the field instance.
        obj = obj1 if (obj1 is not None) else obj2
        field_name = field_name or field
        field_instance = (
            get_field_object(obj, field_name) if (obj and field_name)
            else None
        )

        if field_instance is not None:
            # Binding the field to an object will construct the vocabulary
            # using a factory if necessary.
            self._vocabulary = field_instance.bind(obj).vocabulary

    def _parseField(self, value, filename=None):
        if value is None:
            value = ''
        elif self._vocabulary is not None:
            value = title_or_value(self._vocabulary, value)

        return AsTextDiff._parseField(self, value, filename)

InitializeClass(ChoiceDiff)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from Products.CMFDiffTool.FieldDiff import FieldDiff


class ListDiff(FieldDiff):
    """Text difference"""

    meta_type = 'List Diff'

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        # Return the list as is for diffing
        return value

InitializeClass(ListDiff)



# -*- coding: utf-8 -*-
from .BaseTestCase import BaseATTestCase
from Products.Archetypes import atapi
from Products.ATContentTypes.content.document import ATDocument
from Products.CMFDiffTool.ATCompoundDiff import ATCompoundDiff
from zope.component import adapts
from zope.component import provideAdapter
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class TestATCompoundDiff(BaseATTestCase):
    """Test the portal_diff tool"""

    def testCompoundDiff(self):
        first = self.folder.invokeFactory('Document', 'extended-document')
        first = self.folder[first]
        second = self.folder.invokeFactory('Document', 'extended-document2')
        second = self.folder[second]
        # Change a value
        first.setText('<p>Body1</p>', mimetype='text/html')
        second.setText('<p>Body2</p>', mimetype='text/html')
        fd = ATCompoundDiff(first, second, None)
        # There should be some fields
        self.assertTrue(len(fd))
        for field in fd:
            # We've changed the body
            if field.label == 'label_body_text':
                # We have the correct values
                self.assertEqual(field.oldValue, '<p>Body1</p>')
                self.assertEqual(field.newValue, '<p>Body2</p>')
                # And the correctly assigned diff type
                self.assertEqual(field.meta_type, 'HTML Diff')
            # We've also changed the id
            elif field.label == 'label_short_name':
                self.assertEqual(field.oldValue, 'extended-document')
                self.assertEqual(field.newValue, 'extended-document2')
                self.assertEqual(field.meta_type, 'Lines Diff')
            # The dates will be different, but everything else should
            # be the same
            elif field.label not in ('label_creation_date',
                                     'label_modification_date'):
                self.assertEqual(field.oldValue, field.newValue)

    def testAdaptedObjects(self):
        # Add a schema extended field to an ATDocument and test that
        # the compound diff can read it
        from archetypes.schemaextender.field import ExtensionField
        from archetypes.schemaextender.tests.mocks import Extender
        from archetypes.schemaextender.tests.mocks import IHighlighted

        # custom field class which does not set attributes directly on the
        # content
        class HighlightedField(ExtensionField, atapi.StringField):
            def get(self, instance, **kwargs):
                return IHighlighted.providedBy(instance)

            def getRaw(self, instance, **kwargs):
                return self.get(instance, **kwargs)

            def set(self, instance, value, **kwargs):
                if value and not IHighlighted.providedBy(instance):
                    alsoProvides(instance, IHighlighted)
                elif not value and IHighlighted.providedBy(instance):
                    noLongerProvides(instance, IHighlighted)

        class TestSchemaExtender(Extender):
            adapts(ATDocument)
            fields = [
                HighlightedField('schemaextender_test',
                                 schemata='settings',
                                 widget=atapi.BooleanWidget(
                                     label='Extended',
                                     description=''),
                                 ),
            ]

        """Ensure that tool instances implement the portal_diff interface"""
        provideAdapter(TestSchemaExtender,
                       name=u"archetypes.schemaextender.tests")

        first = self.folder.invokeFactory('Document', 'extended-document')
        first = self.folder[first]
        second = self.folder.invokeFactory('Document', 'extended-document2')
        second = self.folder[second]
        # Change the value
        alsoProvides(second, IHighlighted)
        fd = ATCompoundDiff(first, second, None)
        field = fd[-1]
        self.assertEqual(field.oldValue, False)
        self.assertEqual(field.newValue, True)
        self.assertEqual(field.label, 'Extended')



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from Products.CMFDiffTool.BaseDiff import _getValue
from Products.CMFDiffTool.BaseDiff import BaseDiff

import difflib


class FieldDiff(BaseDiff):
    """Text difference"""

    meta_type = 'Field Diff'

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if filename is None:
            # Since we only want to compare a single field, make a
            # one-item list out of it
            return [value]
        else:
            return [
                self.filenameTitle(filename),
                value
            ]

    def getLineDiffs(self):
        a = self._parseField(self.oldValue, filename=self.oldFilename)
        b = self._parseField(self.newValue, filename=self.newFilename)
        return difflib.SequenceMatcher(None, a, b).get_opcodes()

    def testChanges(self, ob):
        """
        Test the specified object to determine if the change set
        will apply without errors
        """
        value = _getValue(ob, self.field)
        if not self.same and value != self.oldValue:
            raise ValueError('Conflict Error during merge',
                             self.field, value, self.oldValue)

    def applyChanges(self, ob):
        """Update the specified object with the difference"""
        # Simplistic update
        self.testChanges(ob)
        if not self.same:
            setattr(ob, self.field, self.newValue)

    def ndiff(self):
        """Return a textual diff"""
        r = []
        a = self._parseField(self.oldValue, filename=self.oldFilename)
        b = self._parseField(self.newValue, filename=self.newFilename)
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == 'replace':
                plain_replace(a, alo, ahi, b, blo, bhi, r)
            elif tag == 'delete':
                dump('-', a, alo, ahi, r)
            elif tag == 'insert':
                dump('+', b, blo, bhi, r)
            elif tag == 'equal':
                dump(' ', a, alo, ahi, r)
            else:
                raise ValueError('unknown tag %r', tag)
        return '\n'.join(r)

InitializeClass(FieldDiff)


def dump(tag, x, lo, hi, r):
    for i in xrange(lo, hi):
        r.append(tag + ' %s' % x[i])


def plain_replace(a, alo, ahi, b, blo, bhi, r):
    assert alo < ahi and blo < bhi
    # dump the shorter block first -- reduces the burden on short-term
    # memory if the blocks are of very different sizes
    if bhi - blo < ahi - alo:
        dump('+', b, blo, bhi, r)
        dump('-', a, alo, ahi, r)
    else:
        dump('-', a, alo, ahi, r)
        dump('+', b, blo, bhi, r)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from Products.CMFDiffTool.libs import htmldiff
from Products.CMFDiffTool.TextDiff import TextDiff


# Give it a dumb name so it doesn't conflict with all the other html diffs
# around.  This uses Ian Bicking's very nice htmldiff.py from Web Ware for
# Python.
class CMFDTHtmlDiff(TextDiff):
    """Text difference"""

    meta_type = 'HTML Diff'

    def inline_diff(self):
        """Return a specialized diff for HTML"""
        a = '\n'.join(self._parseField(self.oldValue,
                                       filename=self.oldFilename))
        b = '\n'.join(self._parseField(self.newValue,
                                       filename=self.newFilename))
        return htmldiff.htmldiff(a, b)

    def _parseField(self, value, filename=None):
        """Use the field's raw value if available."""
        if value is None:
            value = ''
        else:
            value = getattr(value, 'raw', value)
        return TextDiff._parseField(self, value, filename)

InitializeClass(CMFDTHtmlDiff)



# -*- coding: utf-8 -*-
from .BaseTestCase import BaseDXTestCase
from Products.CMFDiffTool import testing
from Products.CMFDiffTool.choicediff import ChoiceDiff
from Products.CMFDiffTool.choicediff import title_or_value
from Products.CMFDiffTool.interfaces import IDifference


class ChoiceDiffTestCase(BaseDXTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj1',
        )
        self.portal.invokeFactory(
            testing.TEST_CONTENT_TYPE_ID,
            'obj2',
        )

        self.obj1 = self.portal['obj1']
        self.obj2 = self.portal['obj2']

    def test_should_diff_choice_field(self):
        self._test_diff_choice(testing.VOCABULARY_TUPLES[0][0],
                               testing.VOCABULARY_TUPLES[0][0], True)
        self._test_diff_choice(testing.VOCABULARY_TUPLES[0][0],
                               testing.VOCABULARY_TUPLES[1][0], False)
        self._test_diff_choice(testing.VOCABULARY_TUPLES[1][0],
                               testing.VOCABULARY_TUPLES[0][0], False)
        self._test_diff_choice(testing.VOCABULARY_TUPLES[1][0],
                               testing.VOCABULARY_TUPLES[1][0], True)

        self._test_diff_choice(testing.VOCABULARY_TUPLES[0][0], None, False)
        self._test_diff_choice(testing.VOCABULARY_TUPLES[1][0], None, False)
        self._test_diff_choice(None, testing.VOCABULARY_TUPLES[0][0], False)
        self._test_diff_choice(None, testing.VOCABULARY_TUPLES[1][0], False)
        self._test_diff_choice(None, None, True)

    def _test_diff_choice(self, value1, value2, same):
        self.obj1.choice = value1
        self.obj2.choice = value2
        diff = ChoiceDiff(self.obj1, self.obj2, 'choice')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)

        inline_diff = diff.inline_diff()
        if same:
            self.assertFalse(inline_diff)
        else:
            if value1 is not None:
                self.assertTrue(
                    title_or_value(testing.VOCABULARY, value1) in inline_diff)
            if value2 is not None:
                self.assertTrue(
                    title_or_value(testing.VOCABULARY, value2) in inline_diff)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from plone.namedfile import NamedFile
from Products.CMFDiffTool.BinaryDiff import BinaryDiff
from Products.CMFDiffTool.ListDiff import ListDiff
from Products.CMFDiffTool.TextDiff import TextDiff


FILE_FIELD_TYPES = []

try:
    from plone.namedfile import field
    FILE_FIELD_TYPES.extend([field.NamedFile, field.NamedImage])

    if getattr(field, 'HAVE_BLOBS', True):
        FILE_FIELD_TYPES.extend([field.NamedBlobFile, field.NamedBlobImage])
except ImportError:
    pass

FILE_FIELD_TYPES = tuple(FILE_FIELD_TYPES)


def named_file_as_str(f):
    return '' if f is None else '%s (%d bytes)' % (f.filename, len(f.data))


def is_same(old_data, old_filename, new_data, new_filename):
    if old_data != new_data:
        return False

    if (old_filename is not None) and (new_filename is not None):
        return old_filename == new_filename

    return True


class NamedFileBinaryDiff(BinaryDiff):

    def __init__(self, obj1, obj2, field, id1=None, id2=None, field_name=None,
                 field_label=None, schemata=None):

        self.field = field
        self.label = field_label or field
        self.schemata = schemata or 'default'
        self.field_name = field_name or field

        old_field = getattr(obj1, field)
        new_field = getattr(obj2, field)

        self.oldValue = getattr(old_field, 'data', None)
        self.newValue = getattr(new_field, 'data', None)

        self.id1 = id1 or getattr(obj1, 'getId', lambda: None)()
        self.id2 = id2 or getattr(obj2, 'getId', lambda: None)()

        self.oldFilename = getattr(old_field, 'filename', None)
        self.newFilename = getattr(new_field, 'filename', None)

        self.same = is_same(
            self.oldValue, self.oldFilename, self.newValue, self.newFilename)

    def _parseField(self, value, filename=None):
        return [
            '' if (value is None)
            else named_file_as_str(NamedFile(data=value, filename=filename))
        ]

    def inline_diff(self):
        css_class = 'InlineDiff'
        old = self._parseField(self.oldValue, self.oldFilename)[0]
        new = self._parseField(self.newValue, self.newFilename)[0]

        return '' if self.same else self.inlinediff_fmt % (css_class, old, new)

InitializeClass(NamedFileBinaryDiff)


def make_lists_same_length(s1, s2, dummy_element):
    if len(s1) > len(s2):
        s2 += [dummy_element] * (len(s1) - len(s2))
    if len(s2) > len(s1):
        s1 += [dummy_element] * (len(s2) - len(s1))


class NamedFileListDiff(ListDiff):
    """Specialization of `ListDiff` to handle lists of files better."""

    same_fmt = """<div class="%s">%s</div>"""
    inlinediff_fmt = TextDiff.inlinediff_fmt

    def __init__(self, obj1, obj2, field, id1=None, id2=None, field_name=None,
                 field_label=None, schemata=None):
        ListDiff.__init__(self, obj1, obj2, field, id1, id2, field_name,
                          field_label, schemata)
        old_values = list(self.oldValue or [])
        new_values = list(self.newValue or [])

        self.same = True
        if len(old_values) != len(new_values):
            self.same = False
        else:
            for (old, new) in zip(old_values, new_values):
                if not is_same(old.data, old.filename, new.data, new.filename):
                    self.same = False
                    break

    def _parseField(self, value, filename=None):
        value = value or []
        return [named_file_as_str(f) for f in value]

    def inline_diff(self):
        if self.same:
            return None

        css_class = 'InlineDiff'

        old_reprs = self._parseField(self.oldValue, None)
        new_reprs = self._parseField(self.newValue, None)

        old_data = [
            {'repr': repr, 'data': value.data, 'filename': value.filename}
            for (repr, value) in zip(old_reprs, self.oldValue or [])
        ]
        new_data = [
            {'repr': repr, 'data': value.data, 'filename': value.filename}
            for (repr, value) in zip(new_reprs, self.newValue or [])
        ]

        dummy_dict = {'repr': '', 'data': None, 'filename': None}
        make_lists_same_length(old_data, new_data, dummy_dict)

        def is_same_dict(d1, d2):
            return is_same(
                d1['data'], d1['filename'], d2['data'], d2['filename']
            )

        return '\n'.join([
            ((self.same_fmt % (css_class, d_old['repr']))
             if is_same_dict(d_old, d_new) else self.inlinediff_fmt
             % (css_class, d_old['repr'], d_new['repr'])
             ) for (d_old, d_new) in zip(old_data, new_data)])

InitializeClass(NamedFileListDiff)



# -*- coding: utf-8 -*-
from plone.namedfile import NamedFile
from Products.CMFDiffTool import namedfile
from Products.CMFDiffTool.interfaces import IDifference

import unittest


class DummyType(object):
    def __init__(self, files):
        """`files` is a sequence of (data, filename) tuples."""
        self.files = files and [
            NamedFile(data=d, filename=fn) for (d, fn) in files]


class AsTextDiffTestCase(unittest.TestCase):

    def test_should_diff_file_lists_correctly(self):
        self._test_diff_files(
            [('data1', u'filename1')],
            [('data2', u'filename2')],
            False
        )
        self._test_diff_files(
            [('data1', u'filename1'), ('datax', u'filenamex')],
            [('data1', u'filename1'), ('datay', u'filenamey')],
            False
        )
        self._test_diff_files(
            [('data1', u'filename1'), ('datax', u'filenamex')],
            [('datax', u'filenamex'), ('data1', u'filename1')],
            False
        )
        self._test_diff_files(
            [('data1', u'filename1')],
            [('data1', u'filename1'), ('datax', u'filenamex')],
            False
        )
        self._test_diff_files(
            [('data1', u'filename1')],
            [('data1', u'filename1')],
            True
        )
        self._test_diff_files(
            [('data1', u'filename1'), ('datax', u'filenamex')],
            [('data1', u'filename1'), ('datax', u'filenamex')],
            True
        )
        self._test_diff_files(
            [('data1', u'filename1'), ('datax', u'filenamex')],
            None,
            False
        )
        self._test_diff_files(
            [('data1', u'filename1'), ('datax', u'filenamex')],
            [],
            False
        )
        self._test_diff_files(None, None, True)
        self._test_diff_files([], [], True)
        self._test_diff_files([], None, True)

    def _test_diff_files(self, files1, files2, same):
        diff = namedfile.NamedFileListDiff(
            DummyType(files1), DummyType(files2), 'files')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)
        self.assertNotEqual(bool(diff.inline_diff()), same)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from Products.CMFDiffTool.BinaryDiff import BinaryDiff
from Products.CMFDiffTool.CMFDTHtmlDiff import CMFDTHtmlDiff
from Products.CMFDiffTool.FieldDiff import FieldDiff
from Products.CMFDiffTool.interfaces.portal_diff import IDifference
from Products.CMFDiffTool.ListDiff import ListDiff
from Products.CMFDiffTool.TextDiff import TextDiff


AT_FIELD_MAPPING = {'text': 'variable_text',
                    'string': 'variable_text',
                    'datetime': FieldDiff,
                    'file': 'variable_binary',
                    'blob': 'variable_binary',
                    'image': BinaryDiff,
                    'lines': ListDiff,
                    'integer': FieldDiff,
                    'float': FieldDiff,
                    'fixedpoint': FieldDiff,
                    'boolean': FieldDiff,
                    'reference': 'raw:ListDiff'}

AT_EXCLUDED_FIELDS = ('modification_date', )


class ATCompoundDiff:
    '''Text difference'''

    meta_type = 'Compound Diff for AT types'

    def __init__(self, obj1, obj2, field, id1=None, id2=None):
        if not id1:
            id1 = obj1.getId()
        if not id2:
            id2 = obj2.getId()
        self.id1 = id1
        self.id2 = id2

        fields = self.getFields(obj1, obj2)
        self._diffs = self.generateSubDiffs(fields, obj1, obj2)

    def __getitem__(self, index):
        return self._diffs[index]

    def __len__(self):
        return len(self._diffs)

    def __iter__(self):
        return iter(self._diffs)

    def generateSubDiffs(self, fields, obj1, obj2):
        diff_list = []
        for field in fields:
            klass = field['klass']
            diff = klass(obj1, obj2, field['accessor'], id1=self.id1,
                         id2=self.id2,
                         field_name=field['name'],
                         field_label=field['label'],
                         schemata=field['schemata'])
            diff_list.append(diff)
        return diff_list

    def getFields(self, obj1, obj2):
        fields = []
        # Make sure we get the fields ordered by schemata, as in the edit view
        schematas = obj1.Schemata()
        schemata_names = schematas.keys()

        # Put default first and metadata last
        if 'default' in schemata_names and schemata_names[0] != 'default':
            schemata_names.remove('default')
            schemata_names.insert(0, 'default')
        if 'metadata' in schemata_names and schemata_names[-1] != 'metadata':
            schemata_names.remove('metadata')
            schemata_names.insert(-1, 'metadata')

        for schemata_name in schemata_names:
            schema = schematas[schemata_name]
            for field in schema.viewableFields(obj1):
                if (field.type in AT_FIELD_MAPPING and
                        field.getName() not in AT_EXCLUDED_FIELDS):
                    is_primary = getattr(field, 'primary', False)
                    label = field.widget.Label(obj1)
                    diff_type = AT_FIELD_MAPPING[field.type]
                    if IDifference.implementedBy(diff_type):
                        fields.append({'name': field.getName(),
                                       'accessor': field.accessor,
                                       'klass': diff_type,
                                       'primary': is_primary,
                                       'label': label,
                                       'schemata': schemata_name})
                    elif 'raw' in diff_type:
                        #  Handle Fields which diff against the edit accessor
                        diff_name = diff_type.split(':')[1]
                        diff_type = globals()[diff_name]
                        fields.append({'name': field.getName(),
                                       'accessor': field.edit_accessor,
                                       'klass': diff_type,
                                       'primary': is_primary,
                                       'label': label,
                                       'schemata': schemata_name})
                    elif diff_type == 'variable_binary':
                        diff_type = BinaryDiff
                        if ('text/' in field.getContentType(obj1) and
                                'text/' in (obj2.getField(field.getName())
                                            .getContentType(obj2))):
                            diff_type = TextDiff
                        fields.append({'name': field.getName(),
                                       'accessor': field.accessor,
                                       'klass': diff_type,
                                       'primary': is_primary,
                                       'label': label,
                                       'schemata': schemata_name})
                    elif diff_type == 'variable_text':
                        diff_type = TextDiff
                        if ('html' in field.getContentType(obj1) and
                                'html' in (obj2.getField(field.getName())
                                           .getContentType(obj2))):
                            diff_type = CMFDTHtmlDiff
                        fields.append({'name': field.getName(),
                                       'accessor': field.accessor,
                                       'klass': diff_type,
                                       'primary': is_primary,
                                       'label': label,
                                       'schemata': schemata_name})
        return fields

InitializeClass(ATCompoundDiff)



# -*- coding: utf-8 -*-
"""
htmldiff.py
(C) Ian Bicking <ianb@colorstudy.com>

Finds the differences between two HTML files.  *Not* line-by-line
comparison (more word-by-word).

Command-line usage:
  ./htmldiff.py test1.html test2.html

Better results if you use mxTidy first.  The output is HTML.
"""

from difflib import SequenceMatcher
from StringIO import StringIO

import cgi
import re


def htmlEncode(s, esc=cgi.escape):
    return esc(s, 1)

commentRE = re.compile('<!--.*?-->', re.S)
tagRE = re.compile('<.*?>', re.S)
headRE = re.compile('<\s*head\s*>', re.S | re.I)


class HTMLMatcher(SequenceMatcher):

    def __init__(self, source1, source2):
        SequenceMatcher.__init__(self, None, source1, source2)

    def set_seq1(self, a):
        SequenceMatcher.set_seq1(self, self.splitHTML(a))

    def set_seq2(self, b):
        SequenceMatcher.set_seq2(self, self.splitHTML(b))

    def splitTags(self, t):
        result = []
        pos = 0
        while 1:
            match = tagRE.search(t, pos=pos)
            if not match:
                result.append(t[pos:])
                break
            result.append(t[pos:match.start()])
            result.append(match.group(0))
            pos = match.end()
        return result

    def splitWords(self, t):
        return t.strip().split()

    def splitHTML(self, t):
        t = commentRE.sub('', t)
        r = self.splitTags(t)
        result = []
        for item in r:
            if item.startswith('<'):
                result.append(item)
            else:
                result.extend(self.splitWords(item))
        return result

    def htmlDiff(self, addStylesheet=False):
        opcodes = self.get_opcodes()
        a = self.a
        b = self.b
        out = StringIO()
        #  print [o[0] for o in opcodes]
        for tag, i1, i2, j1, j2 in opcodes:
            if tag == 'equal':
                for item in a[i1:i2]:
                    out.write(item)
                    out.write(' ')
            if tag == 'delete' or tag == 'replace':
                self.textDelete(a[i1:i2], out)
            if tag == 'insert' or tag == 'replace':
                self.textInsert(b[j1:j2], out)
        html = out.getvalue()
        out.close()
        if addStylesheet:
            html = self.addStylesheet(html, self.stylesheet())
        return html

    def textDelete(self, lst, out):
        inSpan = False
        for item in lst:
            if item.startswith('<'):
                if inSpan:
                    out.write(self.endDeleteText())
                    inSpan = False
                out.write(self.formatDeleteTag(item))
            else:
                if not inSpan:
                    out.write(self.startDeleteText())
                    inSpan = True
                out.write(item)
                out.write(' ')
        if inSpan:
            out.write(self.endDeleteText())

    def textInsert(self, lst, out):
        inSpan = False
        for item in lst:
            if item.startswith('<'):
                if inSpan:
                    out.write(self.endInsertText())
                    inSpan = False
                out.write(self.formatInsertTag(item))
                out.write(item)
                out.write(' ')
            else:
                if not inSpan:
                    out.write(self.startInsertText())
                    inSpan = True
                out.write(item)
                out.write(' ')
        if inSpan:
            out.write(self.endInsertText())

    def stylesheet(self):
        return '''
.insert { background-color: #aaffaa }
.delete { background-color: #ff8888 }
.tagInsert { background-color: #007700; color: #ffffff }
.tagDelete { background-color: #770000; color: #ffffff }
'''

    def addStylesheet(self, html, ss):
        match = headRE.search(html)
        if match:
            pos = match.end()
        else:
            pos = 0
        return ('{0}<style type="text/css"><!--\n{1}\n--></style>{2}'.format(
            html[:pos], ss, html[pos:]))

    def startInsertText(self):
        return '<span class="insert">'

    def endInsertText(self):
        return '</span> '

    def startDeleteText(self):
        return '<span class="delete">'

    def endDeleteText(self):
        return '</span> '

    def formatInsertTag(self, tag):
        return ('<span class="tagInsert">insert: <tt>%s</tt></span> ' %
                htmlEncode(tag))

    def formatDeleteTag(self, tag):
        return ('<span class="tagDelete">delete: <tt>%s</tt></span> ' %
                htmlEncode(tag))


class NoTagHTMLMatcher(HTMLMatcher):
    def formatInsertTag(self, tag):
        return ''

    def formatDeleteTag(self, tag):
        return ''


def htmldiff(source1, source2, addStylesheet=False):
    """
    Return the difference between two pieces of HTML

        >>> htmldiff('test1', 'test2')
        '<span class="delete">test1 </span> <span class="insert">test2 </span> '  # NOQA
        >>> htmldiff('test1', 'test1')
        'test1 '
        >>> htmldiff('<b>test1</b>', '<i>test1</i>')
        '<span class="tagDelete">delete: <tt>&lt;b&gt;</tt></span> <span class="tagInsert">insert: <tt>&lt;i&gt;</tt></span> <i> test1 <span class="tagDelete">delete: <tt>&lt;/b&gt;</tt></span> <span class="tagInsert">insert: <tt>&lt;/i&gt;</tt></span> </i> '
    """
    h = HTMLMatcher(source1, source2)
    return h.htmlDiff(addStylesheet)


def diffFiles(f1, f2):
    source1 = open(f1).read()
    source2 = open(f2).read()
    return htmldiff(source1, source2, True)


class SimpleHTMLMatcher(HTMLMatcher):
    """
    Like HTMLMatcher, but returns a simpler diff
    """

    def startInsertText(self):
        return '+['

    def endInsertText(self):
        return ']'

    def startDeleteText(self):
        return '-['

    def endDeleteText(self):
        return ']'

    def formatInsertTag(self, tag):
        return '+[%s]' % tag

    def formatDeleteTag(self, tag):
        return '-[%s]' % tag


def simplehtmldiff(source1, source2):
    """
    Simpler form of htmldiff; mostly for testing, like:

        >>> simplehtmldiff('test1', 'test2')
        '-[test1 ]+[test2 ]'
        >>> simplehtmldiff('<b>Hello world!</b>', '<i>Hello you!</i>')
        '-[<b>]+[<i>]<i> Hello -[world! ]-[</b>]+[you! ]+[</i>]</i> '
    """
    h = SimpleHTMLMatcher(source1, source2)
    return h.htmlDiff()


class TextMatcher(HTMLMatcher):

    def set_seq1(self, a):
        SequenceMatcher.set_seq1(self, a.split('\n'))

    def set_seq2(self, b):
        SequenceMatcher.set_seq2(self, b.split('\n'))

    def htmlDiff(self, addStylesheet=False):
        opcodes = self.get_opcodes()
        a = self.a
        b = self.b
        out = StringIO()
        for tag, i1, i2, j1, j2 in opcodes:
            if tag == 'equal':
                self.writeLines(a[i1:i2], out)
            if tag == 'delete' or tag == 'replace':
                out.write(self.startDeleteText())
                self.writeLines(a[i1:i2], out)
                out.write(self.endDeleteText())
            if tag == 'insert' or tag == 'replace':
                out.write(self.startInsertText())
                self.writeLines(b[j1:j2], out)
                out.write(self.endInsertText())
        html = out.getvalue()
        out.close()
        if addStylesheet:
            html = self.addStylesheet(html, self.stylesheet())
        return html

    def writeLines(self, lines, out):
        for line in lines:
            line = htmlEncode(line)
            line = line.replace('  ', '&nbsp; ')
            line = line.replace('\t', '&nbsp; &nbsp; &nbsp; &nbsp; ')
            if line.startswith(' '):
                line = '&nbsp;' + line[1:]
            out.write('<tt>%s</tt><br>\n' % line)

if __name__ == '__main__':
    import sys
    if not sys.argv[1:]:
        print('Usage: %s file1 file2' % sys.argv[0])  # NOQA
        print('or to test: %s test' % sys.argv[0])  # NOQA
    elif sys.argv[1] == 'test' and not sys.argv[2:]:
        import doctest
        doctest.testmod()
    else:
        print(diffFiles(sys.argv[1], sys.argv[2]))  # NOQA



# -*- coding: utf-8 -*-
from plone.app.textfield import RichText
from plone.dexterity.browser.edit import DefaultEditForm
from Products.CMFDiffTool.choicediff import ChoiceDiff
from Products.CMFDiffTool.CMFDTHtmlDiff import CMFDTHtmlDiff
from Products.CMFDiffTool.FieldDiff import FieldDiff
from Products.CMFDiffTool.ListDiff import ListDiff
from Products.CMFDiffTool.namedfile import FILE_FIELD_TYPES
from Products.CMFDiffTool.namedfile import NamedFileBinaryDiff
from Products.CMFDiffTool.namedfile import NamedFileListDiff
from Products.CMFDiffTool.TextDiff import AsTextDiff
from Products.CMFDiffTool.TextDiff import TextDiff
from zope.globalrequest import getRequest
from zope.schema import Bool
from zope.schema import Bytes
from zope.schema import Choice
from zope.schema import Container
from zope.schema import Date
from zope.schema import Datetime
from zope.schema import Iterable
from zope.schema import Text
from zope.schema import Time


# TODO: Perhaps this can be replaced with some kind of Zope 3 style
# adaptation, in order to provide better extensibility.
FIELDS_AND_DIFF_TYPES_RELATION = [
    (FILE_FIELD_TYPES, NamedFileBinaryDiff),
    ((Iterable, Container), ListDiff),
    ((Date, Datetime, Time), AsTextDiff),
    ((Bool, ), AsTextDiff),
    ((Choice, ), ChoiceDiff),
    ((Text, Bytes), TextDiff),
    ((RichText, ), CMFDTHtmlDiff),
]

"""
Relates field types (`zope.schema.Field` subclasses) and "diff types"
(`Products.CMFEditions.BaseDiff.BaseDiff` subclasses).

To find the best diff type for a field type this list will be searched until
a match is found.
If a match is not found then `FALL_BACK_DIFF_TYPE` is used.
"""

FALL_BACK_DIFF_TYPE = FieldDiff

# TODO: Perhaps this is not the best approach. Instead we should write a diff
# type which can handle lists of any kind of elements, providing extensibility
# mechanisms if we want to specialize the handling of certain value types.
# (rafaelbco)
VALUE_TYPES_AND_DIFF_TYPES_RELATION = [
    (FILE_FIELD_TYPES, NamedFileListDiff),
]

"""
When a field is detected to be a list-like field we use this list in the same
fashion as `FIELDS_AND_DIFF_TYPES_RELATION` to try to find the best "diff type
" according to the "value type" of the field, i.e the type of the elements in
the list. If a match is not found then a fall back is used.
"""

# TODO: provide an easier way to exclude fields.
EXCLUDED_FIELDS = ('modification_date', 'IVersionable.changeNote')
"""Names of fields not to compare."""


class DexterityCompoundDiff(object):
    """Same as `Products.CMFDiffTool.ATCompoundDiff.ATCompoundDiff` but for
    Dexterity.
    """

    meta_type = 'Compound Diff for Dexterity types'

    def __init__(self, obj1, obj2, field, id1=None, id2=None):
        self.id1 = id1 or obj1.getId()
        self.id2 = id2 or obj2.getId()
        self.obj1 = obj1
        self._diffs = self._diff(obj1, obj2)

    def __getitem__(self, index):
        return self._diffs[index]

    def __len__(self):
        return len(self._diffs)

    def __iter__(self):
        return iter(self._diffs)

    def _diff(self, obj1, obj2):
        """
        Compute the differences between 2 objects.

        Return: a sequence of `IDifference` objects.
        """

        diffs = []
        for field, field_name in self._compute_fields_order(obj1):
            if field_name not in EXCLUDED_FIELDS:
                schema_name = '.' in field_name and \
                    field_name.split('.')[0] or 'default'
                diffs.append(self._diff_field(obj1, obj2, field, schema_name))

        return diffs

    def _diff_field(self, obj1, obj2, field, schema_name):
        """
        Compute the differences between 2 objects in respect to the given
        field (`zope.schema.Field` instance).

        Return: an `IDifference` object.
        """
        diff_type = self._get_diff_type(field)
        return diff_type(
            obj1,
            obj2,
            field.getName(),
            id1=self.id1,
            id2=self.id2,
            field_name=field.getName(),
            field_label=field.title,
            schemata=schema_name
        )

    def _get_diff_type(self, field):
        """
        Return a subclass of `Products.CMFEditions.BaseDiff.BaseDiff` suitable
        for the given `zope.schema.Field` instance.
        """
        diff_type = self._compute_diff_type(
            field, FIELDS_AND_DIFF_TYPES_RELATION)

        if diff_type is ListDiff:
            return (
                self._compute_diff_type(
                    field.value_type, VALUE_TYPES_AND_DIFF_TYPES_RELATION) or
                diff_type
            )

        return diff_type or FALL_BACK_DIFF_TYPE

    def _compute_diff_type(self, field, relation):
        """
        Return the best "diff type" (subclass of
        `Products.CMFEditions.BaseDiff.BaseDiff`) suitable for the given
        `zope.schema.Field` instance according to `relation`. The `relation`
        is searched until a match is found. Return `None` otherwise.

        Parameters:
        field -- `zope.schema.Field` instance.
        relation -- Sequence of tuples (field_types, diff_type) where
            field_types is a tuple of `zope.schema.Field` subclasses
            and diff_type is a `Products.CMFEditions.BaseDiff.BaseDiff`
            subclass.
        """

        for (field_types, diff_type) in relation:
            if isinstance(field, field_types):
                return diff_type

        return None

    def _compute_fields_order(self, obj):
        """
        Given a content, compute the field ordering the way the edit form does.

        Return: a list of tuples (field, field name) in order.
        """
        form = DefaultEditForm(obj, getRequest())
        form.portal_type = obj.portal_type
        form.updateFields()
        all_fields = list()
        all_fields += [(form.fields[name].field, name) for name in form.fields]
        if form.groups:
            for group in form.groups:
                all_fields += [(group.fields[name].field, name) for name in group.fields]

        return all_fields




# -*- coding: utf-8 -*-
from .BaseTestCase import BaseTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.dexteritydiff import DexterityCompoundDiff


class InstallTestCase(BaseTestCase):

    def test_compound_diff_type_should_be_registered(self):
        diff_tool = getToolByName(self.portal, 'portal_diff')
        self.assertTrue(
            DexterityCompoundDiff.meta_type in diff_tool.listDiffTypes())
        self.assertTrue(
            diff_tool.getDiffType(DexterityCompoundDiff.meta_type))



# -*- coding: utf-8 -*-
#
# ChangeSet.py - Zope object representing the differences between
# objects
#
# Code by Brent Hendricks
#
# (C) 2003 Brent Hendricks - licensed under the terms of the
# GNU General Public License (GPL).  See LICENSE.txt for details

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from Acquisition import Implicit
from ComputedAttribute import ComputedAttribute
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.interfaces import IChangeSet
from zope.interface import implements

import logging


logger = logging.getLogger('CMFDiffTool')


class BaseChangeSet(Implicit):
    """A ChangeSet represents the set of differences between two objects"""

    implements(IChangeSet)
    # This should really not be needed just for same, we should use a method
    __allow_access_to_unprotected_subobjects__ = 1
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """ChangeSet constructor"""
        self.id = id
        self.title = title
        self._diffs = []
        self._added = []
        self._removed = []
        self.ob1_path = []
        self.ob2_path = []
        self._changesets = {}
        self.recursive = 0

    security.declarePublic('getId')  # NOQA

    def getId(self):
        """ChangeSet id"""
        return self.id

    def __getitem__(self, key):
        return self._changesets[key]

    def _isSame(self):
        """Returns true if there are no differences between the two objects"""
        return reduce(lambda x, y: x and y, [d.same for d in self._diffs], 1)

    security.declarePublic('same')  # NOQA
    same = ComputedAttribute(_isSame)

    security.declarePublic('computeDiff')  # NOQA

    def computeDiff(self, ob1, ob2, recursive=1, exclude=None,
                    id1=None, id2=None):
        """Compute the differences from ob1 to ob2 (ie. ob2 - ob1).

        The results can be accessed through getDiffs()"""

        if exclude is None:
            exclude = []

        # Reset state
        self._diffs = []
        self._added = []
        self._removed = []
        self._changed = []
        self._changesets = {}

        purl = getToolByName(self, 'portal_url', None)
        if purl is not None:
            try:
                self.ob1_path = purl.getRelativeContentPath(ob1)
                self.ob2_path = purl.getRelativeContentPath(ob2)
            except AttributeError:
                # one or both of the objects may not have a path
                return
        diff_tool = getToolByName(self, 'portal_diff')
        self._diffs = diff_tool.computeDiff(ob1, ob2, id1=id1, id2=id2)

        if (recursive and
                ob1.isPrincipiaFolderish and
                ob2.isPrincipiaFolderish):
            self.recursive = 1
            ids1 = set(ob1.objectIds())
            ids2 = set(ob2.objectIds())
            self._changed = ids1.intersection(ids2)
            self._removed = ids1.difference(ids2)
            self._added = ids2.difference(ids1)

            # Ignore any excluded items
            for id in exclude:
                try:
                    self._added.remove(id)
                except ValueError:
                    pass
                try:
                    self._removed.remove(id)
                except ValueError:
                    pass
                try:
                    self._changed.remove(id)
                except ValueError:
                    pass

            # Calculate a ChangeSet for every subobject that has changed
            # XXX this is a little strange as self._changed doesn't appear
            # to list changed objects, but rather objects which have been
            # reordered or renamed.
            for id in self._changed:
                self._addSubSet(id, ob1, ob2, exclude, id1, id2)

    def _addSubSet(self, id, ob1, ob2, exclude, id1, id2):
        cs = BaseChangeSet(id, title='Changes to: %s' % id)
        cs = cs.__of__(self)
        cs.computeDiff(ob1[id], ob2[id], exclude=exclude, id1=id1, id2=id2)
        self._changesets[id] = aq_base(cs)

    security.declarePublic('testChanges')  # NOQA

    def testChanges(self, ob):
        """
        Test the specified object to determine if the change set
        will apply without errors
        """
        for d in self._diffs:
            d.testChanges(ob)

        for id in self._changed:
            cs = self[id]
            child = ob[id]
            cs.testChanges(child)

    security.declarePublic('applyChanges')  # NOQA

    def applyChanges(self, ob):
        """Apply the change set to the specified object"""
        for d in self._diffs:
            d.applyChanges(ob)

        if self._removed:
            ob.manage_delObjects(self._removed)

        for id in self._added:
            child = self[id]
            ob.manage_clone(child, id)

        for id in self._changed:
            cs = self[id]
            child = ob[id]
            cs.applyChanges(child)

    security.declarePublic('getDiffs')  # NOQA

    def getDiffs(self):
        """
        Returns the list differences between the two objects.

        Each difference is a single object implementing
        the IDifference interface
        """
        return self._diffs

    security.declarePublic('getSubDiffs')  # NOQA

    def getSubDiffs(self):
        """If the ChangeSet was computed recursively, returns a list
           of ChangeSet objects representing subjects differences

           Each ChangeSet will have the same ID as the objects whose
           difference it represents.
           """
        return [self[id] for id in self._changed]

    security.declarePublic('getAddedItems')  # NOQA

    def getAddedItems(self):
        """If the ChangeSet was computed recursively, returns the list
        of IDs of items that were added.

        A copy of these items is available as a cubject of the ChangeSet
        """
        return list(self._added)

    security.declarePublic('getRemovedItems')  # NOQA

    def getRemovedItems(self):
        """If the ChangeSet was computed recursively, returns the list
        of IDs of items that were removed"""
        return list(self._removed)



# -*- coding: utf-8 -*-
"""CMFDiffTool.py

   Calculate differences between content objects
"""
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from App.class_init import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import registerToolInterface
from Products.CMFCore.utils import UniqueObject
from Products.CMFDiffTool.ChangeSet import BaseChangeSet
from Products.CMFDiffTool.interfaces import IDiffTool
from Products.CMFPlone.utils import safe_hasattr
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from zExceptions import BadRequest
from zope.interface import implements


class CMFDiffTool(UniqueObject, SimpleItem):
    """ """

    id = 'portal_diff'
    meta_type = 'CMF Diff Tool'

    security = ClassSecurityInfo()

    manage_options = (({'label': 'Configure', 'action': 'manage_difftypes'},
                      {'label': 'Overview', 'action': 'manage_overview'},
                       ) + SimpleItem.manage_options
                      )

    implements(IDiffTool)

    #  Internal attributes
    _difftypes = {}

    def __init__(self):
        self._pt_diffs = {}

    security.declareProtected(ManagePortal, 'manage_difftypes')  # NOQA
    manage_difftypes = PageTemplateFile('zpt/editCMFDiffTool', globals())

    def manage_editDiffFields(self, updates, REQUEST=None):
        """Edit the portal type fields"""

        # Clear the old values
        del self._pt_diffs
        self._pt_diffs = {}
        for r in updates:
            if r.get('delete', None):
                continue
            self.setDiffField(r.pt_name, r.field, r.diff)

        self._p_changed = 1

        if REQUEST:
            return self.manage_difftypes(
                manage_tabs_message='Diff mappings updated')

    security.declareProtected(ManagePortal, 'listDiffTypes')  # NOQA

    def manage_addDiffField(self, pt_name, field, diff, REQUEST=None):
        """Add a new diff field from the ZMI"""
        self.setDiffField(pt_name, field, diff)
        if REQUEST:
            return self.manage_difftypes(manage_tabs_message='Field added')

    def setDiffField(self, pt_name, field, diff):
        """
        Set the diff type for 'field' on the portal type 'pt_name' to 'diff'
        """
        if pt_name not in self.portal_types.listContentTypes():
            raise BadRequest('Error: invalid portal type')

        elif not field:
            raise BadRequest('Error: no field specified')

        elif diff not in self.listDiffTypes():
            raise BadRequest('Error: invalid diff type')

        else:
            self._pt_diffs.setdefault(pt_name, {})[field] = diff
            self._p_changed = 1

    #  Interface fulfillment
    security.declareProtected(ManagePortal, 'listDiffTypes')  # NOQA

    def listDiffTypes(self):
        """List the names of the registered difference types"""
        return self._difftypes.keys()

    security.declareProtected(ManagePortal, 'getDiffType')  # NOQA

    def getDiffType(self, diff):
        """Return a class corresponding to the specified diff type.
        Instances of the class will implement the IDifference
        interface"""
        return self._difftypes.get(diff, None)

    security.declareProtected(ManagePortal, 'setDiffForPortalType')  # NOQA

    def setDiffForPortalType(self, pt_name, mapping):
        """Set the difference type(self, s) for the specific portal type

        mapping is a dictionary where each key is an attribute or
        method on the given portal type, and the value is the name of
        a difference type."""
        # FIXME: Do some error checking
        self._pt_diffs[pt_name] = mapping.copy()
        self._p_changed = 1

    security.declareProtected(ManagePortal, 'getDiffForPortalType')  # NOQA

    def getDiffForPortalType(self, pt_name):
        """Returns a dictionary where each key is an attribute or
        method on the given portal type, and the value is the name of
        a difference type."""
        # Return a copy so we don't have to worry about the user changing it
        return self._pt_diffs.get(pt_name, {}).copy()

    security.declarePublic('computeDiff')  # NOQA

    def computeDiff(self, ob1, ob2, id1=None, id2=None):
        """Compute the differences between two objects and return the
        results as a list.  Each object in the list will implement the
        IDifference interface"""

        # Try to get the portal type from obj1 first.  If that fails, use obj2
        pt_name = ''
        try:
            pt_name = aq_base(ob1).portal_type
        except AttributeError:
            try:
                pt_name = aq_base(ob2).portal_type
            except AttributeError:
                pass

        diff_map = self._pt_diffs.get(pt_name, {})

        diffs = []
        for field, klass_name in diff_map.items():
            klass = self._difftypes[klass_name]
            f_diff = klass(ob1, ob2, field, id1=id1, id2=id2)
            # handle compound diff types
            if safe_hasattr(f_diff, '__getitem__'):
                diffs.extend(f_diff)
            else:
                diffs.append(f_diff)
        return diffs

    security.declarePublic('createChangeSet')  # NOQA

    def createChangeSet(self, ob1, ob2, id1=None, id2=None):
        """Returns a ChangeSet object that represents the differences
        between ob1 and ob2 (ie. ob2 - ob1) ."""
        # FIXME: Pick a better ID
        cs = BaseChangeSet('Changes').__of__(self)
        cs.computeDiff(ob1, ob2, id1=id1, id2=id2)
        return aq_base(cs)


def registerDiffType(klass):
    """Register a class for computing differences.

    Instances of the class must implement the IDifference
    interface."""

    CMFDiffTool._difftypes[klass.meta_type] = klass


def unregisterDiffType(klass):
    """Register a class for computing differences.

    Instances of the class must implement the IDifference
    interface."""

    del CMFDiffTool._difftypes[klass.meta_type]

InitializeClass(CMFDiffTool)
registerToolInterface('portal_diff', IDiffTool)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from os import linesep
from Products.CMFDiffTool import CMFDiffToolMessageFactory as _
from Products.CMFDiffTool.FieldDiff import FieldDiff
from Products.CMFDiffTool.utils import safe_unicode
from Products.CMFDiffTool.utils import safe_utf8
from zope.component.hooks import getSite

import difflib


class TextDiff(FieldDiff):
    """Text difference"""

    meta_type = 'Lines Diff'
    inlinediff_fmt = """
<div class="%s">
    <del>%s</del>
    <ins>%s</ins>
</div>
"""

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if value is None:
            value = ''
        if filename is None:
            # Split the text into a list for diffs
            return value.splitlines()
        else:
            return [self.filenameTitle(filename)] + value.splitlines()

    def unified_diff(self):
        """Return a unified diff"""
        a = [safe_utf8(i) for i in
             self._parseField(self.oldValue, filename=self.oldFilename)]
        b = [safe_utf8(i) for i in
             self._parseField(self.newValue, filename=self.newFilename)]
        return linesep.join(difflib.unified_diff(a, b, self.id1, self.id2))

    def html_diff(self, context=True, wrapcolumn=40):
        """Return an HTML table showing differences"""
        # difflib is not Unicode-aware, so we need to force everything to
        # utf-8 manually
        a = [safe_unicode(i) for i in
             self._parseField(self.oldValue, filename=self.oldFilename)]
        b = [safe_unicode(i) for i in
             self._parseField(self.newValue, filename=self.newFilename)]
        vis_diff = difflib.HtmlDiff(wrapcolumn=wrapcolumn)
        diff = safe_utf8(vis_diff.make_table(a, b,
                                             safe_unicode(self.id1),
                                             safe_unicode(self.id2),
                                             context=context))
        return diff

    def inline_diff(self):
        """Simple inline diff that just assumes that either the filename
        has changed, or the text has been completely replaced."""
        css_class = 'InlineDiff'
        old_attr = self._parseField(self.oldValue,
                                    filename=self.oldFilename)
        new_attr = self._parseField(self.newValue,
                                    filename=self.newFilename)
        if old_attr:
            old_fname = old_attr.pop(0)
        else:
            old_fname = None
        if new_attr:
            new_fname = new_attr.pop(0)
        else:
            new_fname = None
        a = linesep.join(old_attr or [])
        b = linesep.join(new_attr or [])
        html = []
        if old_fname != new_fname:
            html.append(
                self.inlinediff_fmt % ('%s FilenameDiff' % css_class,
                                       old_fname, new_fname)
            )
        if a != b:
            html.append(
                self.inlinediff_fmt % (css_class, a, b)
            )
        if html:
            return linesep.join(html)

InitializeClass(TextDiff)


class AsTextDiff(TextDiff):
    """
    Specialization of `TextDiff` that converts any value to text in order to
    provide an inline diff visualization. Also translated (i18n) the
    strings `True` and `False`.
    """

    def _parseField(self, value, filename=None):
        if value is None:
            value = ''

        # In tests translation is not available, so we account for this
        # case here.
        translate = getattr(getSite(), 'translate', None)
        if translate is not None:
            value = translate(_(value))

        return TextDiff._parseField(self, safe_unicode(value), filename)

InitializeClass(AsTextDiff)



# -*- coding: utf-8 -*-
"""Initialize CMFDiffTool Product"""
# Set up a MessageFactory for the cmfdifftool domain
from zope.i18nmessageid import MessageFactory
CMFDiffToolMessageFactory = MessageFactory('cmfdifftool')

from Products.CMFCore.utils import ToolInit  # NOQA
from Products.CMFDiffTool import CMFDiffTool  # NOQA
from Products.CMFDiffTool import FieldDiff  # NOQA
from Products.CMFDiffTool import TextDiff  # NOQA
from Products.CMFDiffTool import ListDiff  # NOQA
from Products.CMFDiffTool import BinaryDiff  # NOQA
from Products.CMFDiffTool import CMFDTHtmlDiff  # NOQA
from Products.CMFDiffTool import ATCompoundDiff  # NOQA


CMFDiffTool.registerDiffType(BinaryDiff.BinaryDiff)
CMFDiffTool.registerDiffType(FieldDiff.FieldDiff)
CMFDiffTool.registerDiffType(ListDiff.ListDiff)
CMFDiffTool.registerDiffType(TextDiff.TextDiff)
CMFDiffTool.registerDiffType(TextDiff.AsTextDiff)
CMFDiffTool.registerDiffType(CMFDTHtmlDiff.CMFDTHtmlDiff)
CMFDiffTool.registerDiffType(ATCompoundDiff.ATCompoundDiff)

# Soft plone.namedfile dependency
try:
    from Products.CMFDiffTool import namedfile
except ImportError:
    pass
else:
    CMFDiffTool.registerDiffType(namedfile.NamedFileBinaryDiff)
    CMFDiffTool.registerDiffType(namedfile.NamedFileListDiff)

# Soft Dexterity dependency
try:
    from Products.CMFDiffTool import dexteritydiff
    from Products.CMFDiffTool import choicediff
except ImportError:
    pass
else:
    CMFDiffTool.registerDiffType(choicediff.ChoiceDiff)
    CMFDiffTool.registerDiffType(dexteritydiff.DexterityCompoundDiff)

tools = (CMFDiffTool.CMFDiffTool, )


def initialize(context):
    ToolInit('CMF Diff Tool',
             tools=tools,
             icon='tool.gif'
             ).initialize(context)



# -*- coding: utf-8 -*-
#
# CMFDiffTool tests
#
from Acquisition import aq_base
from os import linesep
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING  # NOQA
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.ChangeSet import BaseChangeSet
from Products.CMFPlone.utils import safe_hasattr
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import TestCase


class TestChangeSet(TestCase):
    """Tests for ChangeSet objects"""

    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.folder = self.portal
        self.p_diff = getToolByName(self.portal, 'portal_diff')
        cs = BaseChangeSet('my_changeset')
        # ChangeSet needs an acquisition wrapper
        self.cs = cs.__of__(self.portal)
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces import IChangeSet
        self.assertTrue(IChangeSet.implementedBy(BaseChangeSet))

    def setupTestObjects(self):
        self.folder.invokeFactory('Document', 'doc1', title='My Title')
        self.folder.manage_pasteObjects(
            self.folder.manage_copyObjects(['doc1']))

    def setupTestFolders(self):
        self.folder.invokeFactory('Folder', 'folder1', title='My Folder Title')
        self.folder.folder1.invokeFactory('Document', 'doc1',
                                          title='My Title1')
        self.folder.folder1.invokeFactory('Document', 'doc2',
                                          title='My Title2')
        self.folder.folder1.invokeFactory('Document', 'doc3',
                                          title='My Title3')
        self.folder.manage_pasteObjects(
            self.folder.manage_copyObjects(['folder1']))

    def testChangeSetUnchanged(self):
        self.setupTestObjects()
        self.cs.computeDiff(self.folder.doc1, self.folder.copy_of_doc1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 15)
        self.assertTrue(diffs[0].same)

    def testChangeSetChanged(self):
        self.setupTestObjects()
        self.folder.copy_of_doc1.setTitle('My New Title')
        self.cs.computeDiff(self.folder.doc1, self.folder.copy_of_doc1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 15)
        self.assertFalse(diffs[0].same)
        self.assertEqual(diffs[0].ndiff(),
                         '- My Title%s+ My New Title' % linesep)

    def testChangeSetFolderUnchanged(self):
        self.setupTestFolders()
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            self.assertTrue(sub_cs[0].same)
            self.assertTrue(sub_cs[1].same)
            self.assertTrue(sub_cs[2].same)

    def testChangeSetFolderChanged(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.setTitle('My New Title')
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertFalse(diffs[0].same)
        self.assertEqual(diffs[0].ndiff(),
                         '- My Folder Title%s+ My New Title' % linesep)
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            self.assertTrue(sub_diffs[0].same)

    def testChangeSetFolderDocChanged(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.doc1.setTitle('My New Title')
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            # doc1 has changed
            if sub_cs[i].getId() == 'doc1':
                self.assertFalse(sub_diffs[0].same)
                self.assertEqual(sub_diffs[0].ndiff(),
                                 '- My Title1%s+ My New Title' % linesep)
            else:
                self.assertTrue(sub_diffs[0].same)

    def testChangeSetFolderDocRemoved(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.manage_delObjects('doc1')
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        sub_cs = self.cs.getSubDiffs()
        # We only have two potentially changed objects
        self.assertEqual(len(sub_cs), 2)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            self.assertTrue(sub_diffs[0].same)
        self.assertFalse(self.cs._added)
        self.assertEqual(list(self.cs._removed), ['doc1'])

    def testChangeSetFolderDocAdded(self):
        self.setupTestFolders()
        self.folder.copy_of_folder1.invokeFactory('Document', 'doc4',
                                                  title='My Doc Title')
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertTrue(diffs[1].same)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            self.assertTrue(sub_diffs[0].same)
        self.assertFalse(self.cs._removed)
        self.assertEqual(list(self.cs._added), ['doc4'])

    def testChangeSetFolderReordered(self):
        self.setupTestFolders()
        if safe_hasattr(aq_base(self.folder.copy_of_folder1),
                        'moveObjectsToTop'):
            self.folder.copy_of_folder1.moveObjectsToTop(['doc3'])
        elif safe_hasattr(aq_base(self.folder.copy_of_folder1),
                          'moveObjectsByDelta'):
            self.folder.copy_of_folder1.moveObjectsByDelta(['doc3'], -3)
        else:
            # We don't have an orderable folder give up
            return
        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertTrue(diffs[0].same)
        self.assertFalse(self.cs._added)
        self.assertFalse(self.cs._removed)
        sub_cs = self.cs.getSubDiffs()
        self.assertEqual(len(sub_cs), 3)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            self.assertTrue(sub_diffs[0].same)
        # XXX we need an explicit way of noting reorders

    def testChangeSetFolderComplex(self):
        self.setupTestFolders()
        # Add a new sub object
        self.folder.copy_of_folder1.invokeFactory('Document', 'doc4',
                                                  title='My Doc Title')
        # Delete a sub object
        self.folder.copy_of_folder1.manage_delObjects('doc2')
        # Change one object
        self.folder.copy_of_folder1.doc3.setTitle('My New Title')
        # Change the folder itself
        self.folder.copy_of_folder1.setTitle('My New Title')
        # Move the changed object
        if safe_hasattr(aq_base(self.folder.copy_of_folder1),
                        'moveObjectsToTop'):
            self.folder.copy_of_folder1.moveObjectsToTop(['doc3'])
        elif safe_hasattr(aq_base(self.folder.copy_of_folder1),
                          'moveObjectsByDelta'):
            self.folder.copy_of_folder1.moveObjectsByDelta(['doc3'], -3)
        else:
            # We don't have an orderable folder give up
            return

        self.cs.computeDiff(self.folder.folder1, self.folder.copy_of_folder1)
        diffs = self.cs.getDiffs()
        self.assertEqual(len(diffs), 14)
        self.assertFalse(diffs[0].same)
        self.assertEqual(diffs[0].ndiff(),
                         '- My Folder Title%s+ My New Title' % linesep)
        self.assertEqual(list(self.cs._added), ['doc4'])
        self.assertEqual(list(self.cs._removed), ['doc2'])
        sub_cs = self.cs.getSubDiffs()
        # We only have two potentially changed objects
        self.assertEqual(len(sub_cs), 2)
        # The sub diffs should show no changes
        for i in range(len(sub_cs)):
            self.assertTrue(isinstance(sub_cs[i], BaseChangeSet))
            sub_diffs = sub_cs[i].getDiffs()
            self.assertEqual(len(sub_diffs), 15)
            if sub_cs[i].getId() == 'doc3':
                self.assertFalse(sub_diffs[0].same)
                self.assertEqual(sub_diffs[0].ndiff(),
                                 '- My Title3%s+ My New Title' % linesep)
            else:
                self.assertTrue(sub_diffs[1].same)
        # XXX we need an explicit way of noting reorders



# -*- coding: utf-8 -*-
from datetime import date
from Products.CMFDiffTool.interfaces import IDifference
from Products.CMFDiffTool.TextDiff import AsTextDiff

import unittest


class DateDummyType(object):
    def __init__(self, date):
        self.date = date


class BoolDummyType(object):
    def __init__(self, bool_field):
        self.bool_field = bool_field


class AsTextDiffTestCase(unittest.TestCase):

    def test_should_diff_anything_as_text(self):
        self._test_diff_date(date(2011, 1, 1), date(2012, 2, 2), False)
        self._test_diff_date(date(2011, 1, 1), date(2011, 1, 1), True)
        self._test_diff_date(date(2011, 1, 1), None, False)
        self._test_diff_date(None, None, True)

    def _test_diff_date(self, d1, d2, same):
        diff = AsTextDiff(DateDummyType(d1), DateDummyType(d2), 'date')
        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)

        inline_diff = diff.inline_diff()
        if same:
            self.assertFalse(inline_diff)
        else:
            self.assertTrue(inline_diff)
            if d1 is not None:
                self.assertTrue(str(d1) in inline_diff)
            if d2 is not None:
                self.assertTrue(str(d2) in inline_diff)
            # self.assertFalse('None' in inline_diff)

    def test_should_diff_boolean_fields(self):
        self._test_diff_bool(False, False, True)
        self._test_diff_bool(False, True, False)
        self._test_diff_bool(True, False, False)
        self._test_diff_bool(True, True, True)
        self._test_diff_bool(False, None, False)
        self._test_diff_bool(True, None, False)
        self._test_diff_bool(None, False, False)
        self._test_diff_bool(None, True, False)
        self._test_diff_bool(None, None, True)

    def _test_diff_bool(self, b1, b2, same):
        diff = AsTextDiff(BoolDummyType(b1), BoolDummyType(b2), 'bool_field')

        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, same)

        inline_diff = diff.inline_diff()
        if same:
            self.assertFalse(inline_diff)
        else:
            self.assertTrue(
                ('True' in inline_diff) or ('False' in inline_diff))
            # self.assertFalse('None' in inline_diff)



# -*- coding: utf-8 -*-
#
# CMFDiffTool tests
#

from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from Products.CMFDiffTool.CMFDiffTool import registerDiffType
from Products.CMFDiffTool.CMFDiffTool import unregisterDiffType
from unittest import TestCase
from zExceptions import BadRequest


class DummyDiff:
    meta_type = 'Dummy Diff Type'


class DummyDiff2:
    meta_type = 'Second Dummy Diff Type'


class TestDiffTool(TestCase):
    """Test the portal_diff tool"""

    layer = PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.p_diff = getToolByName(self.layer['portal'], 'portal_diff')
        # clear pt_diff registry
        self.p_diff._pt_diffs = {}

        # patch portal_types to list `Document` in the listContentTypes
        # a plausability check is done in the `setDiffForPortalType` method
        # but we have no content registry
        portal_types = getToolByName(self.layer['portal'], 'portal_types')
        self._old_listContentTypes = portal_types.listContentTypes
        portal_types.listContentTypes = lambda: ['Document']
        registerDiffType(DummyDiff)

    def tearDown(self):
        portal_types = getToolByName(self.layer['portal'], 'portal_types')
        portal_types.listContentTypes = self._old_listContentTypes

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces.portal_diff import portal_diff
        self.assertTrue(portal_diff.providedBy(self.p_diff))

    def testRegisterDiffType(self):
        """Test registration of Diff types"""
        unregisterDiffType(DummyDiff)
        self.assertNotIn('Dummy Diff Type', self.p_diff.listDiffTypes())
        registerDiffType(DummyDiff)
        self.assertIn('Dummy Diff Type', self.p_diff.listDiffTypes())

    def testSetDiff(self):
        """Test setDiffForPortalType() method"""
        d = {'field1': 'TestDiff', 'field2': 'Dummy Diff Type'}
        self.p_diff.setDiffForPortalType('Document', d)
        self.assertEqual(self.p_diff.getDiffForPortalType('Document'), d)

    def testSetDiffReplaces(self):
        """Test that setDiffForPortalType() replaces old data"""
        d1 = {'field1': 'TestDiff', 'field2': 'Dummy Diff Type'}
        d2 = {'field3': 'Dummy Diff Type'}
        self.p_diff.setDiffForPortalType('Document', d1)
        self.p_diff.setDiffForPortalType('Document', d2)
        self.assertEqual(self.p_diff.getDiffForPortalType('Document'), d2)

    def testSingleSetDiffField(self):
        """Test setDiffField method"""
        self.p_diff.setDiffField('Document', 'title', 'Dummy Diff Type')
        self.assertEqual(self.p_diff.getDiffForPortalType('Document'),
                         {'title': 'Dummy Diff Type'})

    def testMultipleSetDiffField(self):
        """
        Test setDiffField method adding a second field to one content type
        """
        self.p_diff.setDiffField('Document', 'title', 'Dummy Diff Type')
        self.p_diff.setDiffField('Document', 'description', 'Dummy Diff Type')
        d = {'title': 'Dummy Diff Type', 'description': 'Dummy Diff Type'}
        self.assertEqual(self.p_diff.getDiffForPortalType('Document'), d)

    def testReplaceSetDiffField(self):
        """Test that setDiffField does a replace for existing fields"""
        registerDiffType(DummyDiff2)
        self.p_diff.setDiffField('Document', 'title', 'Dummy Diff Type')
        self.p_diff.setDiffField('Document', 'title', 'Second Dummy Diff Type')
        d = {'title': 'Second Dummy Diff Type'}
        self.assertEqual(self.p_diff.getDiffForPortalType('Document'), d)
        unregisterDiffType(DummyDiff2)

    def testSetDiffFieldNameFailure(self):
        self.assertRaises(BadRequest, self.p_diff.setDiffField,
                          'Bob', 'title', 'Dummy Diff Type')

    def testSetDiffFieldBlankFieldFailure(self):
        self.assertRaises(BadRequest, self.p_diff.setDiffField,
                          'Document', '', 'Dummy Diff Type')

    def testSetDiffFieldInvalidDiffFailure(self):
        self.assertRaises(BadRequest, self.p_diff.setDiffField,
                          'Document', 'title', 'NoDiff')

    def beforeTearDown(self):
        # Undo changes that don't get rolled back (i.e. module level changes)
        unregisterDiffType(DummyDiff)



# -*- coding: utf-8 -*-
from App.class_init import InitializeClass
from os import linesep
from Products.CMFDiffTool.BaseDiff import _getValue
from Products.CMFDiffTool.FieldDiff import FieldDiff


class BinaryDiff(FieldDiff):
    """Simple binary difference"""

    meta_type = 'Binary Diff'
    inlinediff_fmt = """
<div class="%s">
    <del>%s</del>
    <ins>%s</ins>
</div>
"""

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if filename is None:
            # Since we only want to compare the filename for
            # binary files, return an empty list
            return []
        else:
            return [self.filenameTitle(filename)]

    def testChanges(self, ob):
        """
        Test the specified object to determine if the change set will
        apply without errors
        """
        value = _getValue(ob, self.field)
        if not self.same and value != self.oldValue:
            raise ValueError('Conflict Error during merge',
                             self.field, value, self.oldValue)

    def applyChanges(self, ob):
        """Update the specified object with the difference"""
        # Simplistic update
        self.testChanges(ob)
        if not self.same:
            setattr(ob, self.field, self.newValue)

    def inline_diff(self):
        """Simple inline diff that just checks that the filename
        has changed."""
        css_class = 'FilenameDiff'
        html = []
        if self.oldFilename != self.newFilename:
            html.append(
                self.inlinediff_fmt % (css_class,
                                       self.filenameTitle(self.oldFilename),
                                       self.filenameTitle(self.newFilename))
            )

        if html:
            return linesep.join(html)

InitializeClass(BinaryDiff)



# -*- coding: utf-8 -*-

from os import linesep
from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFDiffTool.TextDiff import TextDiff
from unittest import TestCase

import sys


_marker = []


class A:
    attribute = 'कामसूत्र'

    def method(self):
        return 'method कामसूत्र'


class B:
    attribute = '過労死'

    def method(self):
        return 'method 過労死'


class TestTextDiff(TestCase):
    """Test the TextDiff class"""
    layer = PLONE_INTEGRATION_TESTING

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces import IDifference
        self.assertTrue(IDifference.implementedBy(TextDiff))

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        fd = TextDiff(a, a, 'attribute')
        self.assertTrue(fd.same)

    def testMethodSame(self):
        """Test method with same value"""
        a = A()
        fd = TextDiff(a, a, 'method')
        self.assertTrue(fd.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, 'attribute')
        self.assertFalse(fd.same)

    def testMethodDiff(self):
        """Test method with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, 'method')
        self.assertFalse(fd.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        fd = TextDiff(a, a, 'attribute')
        expected = [('equal', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        fd = TextDiff(a, b, 'attribute')
        expected = [('replace', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with same value"""
        a = A()
        fd = TextDiff(a, a, 'attribute')
        self.assertEqual(fd.ndiff(), '  कामसूत्र')

    def testDiffText(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        expected = '- कामसूत्र%s+ 過労死' % linesep
        fd = TextDiff(a, b, 'attribute')
        self.assertEqual(fd.ndiff(), expected)

    def testUnifiedDiff(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        if sys.version_info[:2] >= (2, 7):
            expected = """--- None

+++ None

@@ -1 +1 @@

-कामसूत्र
+過労死"""
        else:
            expected = """--- None

+++ None

@@ -1,1 +1,1 @@

-कामसूत्र
+過労死"""
        fd = TextDiff(a, b, 'attribute')
        self.assertEqual(fd.unified_diff(), expected)

    def testHTMLDiff(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        expected = """
    <table class="diff" id="difflib_chg_to0__top"
           cellspacing="0" cellpadding="0" rules="groups" >
        <colgroup></colgroup> <colgroup></colgroup> <colgroup></colgroup>
        <colgroup></colgroup> <colgroup></colgroup> <colgroup></colgroup>
        <thead><tr><th class="diff_next"><br /></th><th colspan="2" class="diff_header">None</th><th class="diff_next"><br /></th><th colspan="2" class="diff_header">None</th></tr></thead>
        <tbody>
            <tr><td class="diff_next" id="difflib_chg_to0__0"><a href="#difflib_chg_to0__top">t</a></td><td class="diff_header" id="from0_1">1</td><td nowrap="nowrap"><span class="diff_sub">कामसूत्र</span></td><td class="diff_next"><a href="#difflib_chg_to0__top">t</a></td><td class="diff_header" id="to0_1">1</td><td nowrap="nowrap"><span class="diff_add">過労死</span></td></tr>
        </tbody>
    </table>"""  # NOQA
        fd = TextDiff(a, b, 'attribute')
        self.assertEqual(fd.html_diff(), expected)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = '3.0.4'

setup(
    name='Products.CMFDiffTool',
    version=version,
    description="Diff tool for Plone",
    long_description=(open("README.rst").read() + "\n" +
                      open("CHANGES.rst").read()),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 2.7",
        "Framework :: Plone",
        "Framework :: Plone :: 5.0",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords='Diff Plone',
    author='Brent Hendricks',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.python.org/pypi/Products.CMFDiffTool',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['Products'],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            'zope.component',
            'plone.app.testing',
            'plone.namedfile',
            'plone.app.dexterity',
            'plone.app.contenttypes',
            'plone.app.robotframework',  # Only because plone.app.event fails
            ]
    ),
    install_requires=[
        'setuptools',
        'zope.interface',
        'Products.CMFCore',
        'Products.GenericSetup',
        'Acquisition',
        'Zope2',
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
    )



# -*- coding: utf-8 -*-
#
# CMFDiffTool tests
#
from os import linesep
from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFDiffTool.ListDiff import ListDiff
from unittest import TestCase


_marker = []


class A:
    attribute = [1, 2, 3]


class B:
    attribute = [1, 2, 3, 4]


class TestListDiff(TestCase):
    """Test the ListDiff class"""

    layer = PLONE_INTEGRATION_TESTING

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces.portal_diff import IDifference
        self.assertTrue(IDifference.implementedBy(ListDiff))

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        diff = ListDiff(a, a, 'attribute')
        self.assertTrue(diff.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        diff = ListDiff(a, b, 'attribute')
        self.assertFalse(diff.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        diff = ListDiff(a, a, 'attribute')
        expected = [('equal', 0, 3, 0, 3)]
        self.assertEqual(diff.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        diff = ListDiff(a, b, 'attribute')
        expected = [('equal', 0, 3, 0, 3), ('insert', 3, 3, 3, 4)]
        self.assertEqual(diff.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with no diff"""
        a = A()
        diff = ListDiff(a, a, 'attribute')
        expected = '  1%(linesep)s  2%(linesep)s  3' % {'linesep': linesep}
        self.assertEqual(diff.ndiff(), expected)

    def testDiffText(self):
        """Test text diff output with no diff"""
        a = A()
        b = B()
        expected = '  1%(linesep)s  2%(linesep)s  3%(linesep)s+ 4' % \
                   {'linesep': linesep}
        diff = ListDiff(a, b, 'attribute')
        self.assertEqual(diff.ndiff(), expected)

        # FIXME: need tests for other kinds of diffs



# -*- coding: utf-8 -*-
#
# CMFDiffTool tests
#
from os import linesep
from plone.app.testing import PLONE_INTEGRATION_TESTING
from Products.CMFDiffTool.FieldDiff import dump
from Products.CMFDiffTool.FieldDiff import FieldDiff
from unittest import TestCase


_marker = []


class A:
    attribute = 'value'

    def method(self):
        return 'method value'


class B:
    attribute = 'different value'

    def method(self):
        return 'different method value'


class U:
    attribute = u"\xfcnicode value"

    def method(self):
        return u"different method val\xfce"


class TestFieldDiff(TestCase):
    """Test the FieldDiff class"""

    layer = PLONE_INTEGRATION_TESTING

    def testInterface(self):
        """Ensure that tool instances implement the portal_diff interface"""
        from Products.CMFDiffTool.interfaces.portal_diff import IDifference
        self.assertTrue(IDifference.implementedBy(FieldDiff))

    def testAttributeSame(self):
        """Test attribute with same value"""
        a = A()
        fd = FieldDiff(a, a, 'attribute')
        self.assertTrue(fd.same)
        uu = U()
        fd = FieldDiff(uu, uu, 'attribute')
        self.assertTrue(fd.same)

    def testMethodSame(self):
        """Test method with same value"""
        a = A()
        fd = FieldDiff(a, a, 'method')
        self.assertTrue(fd.same)
        uu = U()
        fd = FieldDiff(uu, uu, 'method')
        self.assertTrue(fd.same)

    def testAttributeDiff(self):
        """Test attribute with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, 'attribute')
        self.assertFalse(fd.same)
        fd = FieldDiff(a, uu, 'attribute')
        self.assertFalse(fd.same)

    def testMethodDiff(self):
        """Test method with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, 'method')
        self.assertFalse(fd.same)
        fd = FieldDiff(a, uu, 'method')
        self.assertFalse(fd.same)

    def testGetLineDiffsSame(self):
        """test getLineDiffs() method with same value"""
        a = A()
        fd = FieldDiff(a, a, 'attribute')
        expected = [('equal', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)
        uu = U()
        fd = FieldDiff(uu, uu, 'attribute')
        expected = [('equal', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testGetLineDiffsDifferent(self):
        """test getLineDiffs() method with different value"""
        a = A()
        b = B()
        uu = U()
        fd = FieldDiff(a, b, 'attribute')
        expected = [('replace', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)
        fd = FieldDiff(a, uu, 'attribute')
        expected = [('replace', 0, 1, 0, 1)]
        self.assertEqual(fd.getLineDiffs(), expected)

    def testSameText(self):
        """Test text diff output with same value"""
        a = A()
        fd = FieldDiff(a, a, 'attribute')
        self.assertEqual(fd.ndiff(), '  value')
        uu = U()
        fd = FieldDiff(uu, uu, 'attribute')
        self.assertEqual(fd.ndiff(), u'  \xfcnicode value')

    def testDiffText(self):
        """Test text diff output with different value"""
        a = A()
        b = B()
        uu = U()
        expected = '- value%s+ different value' % linesep
        fd = FieldDiff(a, b, 'attribute')
        self.assertEqual(fd.ndiff(), expected)
        expected = u"- value%s+ \xfcnicode value" % linesep
        fd = FieldDiff(a, uu, 'attribute')
        self.assertEqual(fd.ndiff(), expected)

    def test_dump_text(self):
        """Test dumping a diff of text."""
        diff = []
        dump('-', ['support'], 0, 1, diff)
        self.assertEqual(diff, ['- support'])
        # Try unicode, a 'u' with an umlaut.
        diff = []
        dump('+', [u's\xfcpport'], 0, 1, diff)
        self.assertEqual(diff, [u'+ s\xfcpport'])
        # Combine them.
        diff = []
        dump('-', ['support'], 0, 1, diff)
        dump('+', [u's\xfcpport'], 0, 1, diff)
        self.assertEqual(diff, ['- support', u'+ s\xfcpport'])

    def test_dump_integer(self):
        """Test dumping a diff of integers."""
        diff = []
        dump('-', [4], 0, 1, diff)
        self.assertEqual(diff, ['- 4'])
        dump('+', [42], 0, 1, diff)
        self.assertEqual(diff, ['- 4', '+ 42'])

    def test_dump_float(self):
        """Test dumping a diff of floats."""
        diff = []
        dump('-', [1.1], 0, 1, diff)
        self.assertEqual(diff, ['- 1.1'])
        dump('+', [1.2], 0, 1, diff)
        self.assertEqual(diff, ['- 1.1', '+ 1.2'])

    def test_dump_boolean(self):
        """Test dumping a diff of booleans."""
        diff = []
        dump('-', [True], 0, 1, diff)
        self.assertEqual(diff, ['- True'])
        dump('+', [False], 0, 1, diff)
        self.assertEqual(diff, ['- True', '+ False'])



# -*- coding: utf-8 -*-
from plone.app.textfield.value import RichTextValue
from Products.CMFDiffTool.CMFDTHtmlDiff import CMFDTHtmlDiff
from Products.CMFDiffTool.interfaces import IDifference

import unittest


class DummyType(object):
    def __init__(self, body):
        self.body = body


class RichTextDiffTestCase(unittest.TestCase):
    """Test RichTextDiff"""

    def test_parseField_value_is_none(self):
        value = None
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), 'body')
        self.assertEqual(diff._parseField(value), [])

    def test_parseField_value_is_not_none(self):
        value = RichTextValue(u'foo')
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), 'body')
        self.assertEqual(diff._parseField(value), [u'foo'])

    def test_inline_diff_same(self):
        value = RichTextValue(u'foo')
        diff = CMFDTHtmlDiff(DummyType(value), DummyType(value), 'body')
        inline_diff = diff.inline_diff()

        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, True)
        self.assertEqual(inline_diff, u'foo ')

    def test_inline_diff_different(self):
        old_value = RichTextValue(u'foo')
        new_value = RichTextValue(u'foo bar')
        diff = CMFDTHtmlDiff(
            DummyType(old_value), DummyType(new_value), 'body')

        inline_diff = diff.inline_diff()

        self.assertTrue(IDifference.providedBy(diff))
        self.assertEqual(diff.same, False)
        self.assertEqual(inline_diff, u'foo <span class="insert">bar </span> ')



# -*- coding: utf-8 -*-
# BaseTestCase

from plone.app.testing.bbb import PloneTestCase
from Products.CMFDiffTool.testing import CMFDiffToolATLayer
from Products.CMFDiffTool.testing import CMFDiffToolDXLayer
from Products.CMFDiffTool.testing import CMFDiffToolLayer


class BaseTestCase(PloneTestCase):
    """ A base testing class for CMFDiffTool

        It includes a layer which installes the product
        and some testing dependencies in a Plone site.
    """
    layer = CMFDiffToolLayer


class BaseATTestCase(PloneTestCase):
    """ A base testing class for CMFDiffTool

        It includes a layer which installes the product
        and some testing dependencies in a Plone site.
    """
    layer = CMFDiffToolATLayer


class BaseDXTestCase(PloneTestCase):
    """ A base testing class for CMFDiffTool

        It includes a layer which installes the product
        and some testing dependencies in a Plone site.
    """
    layer = CMFDiffToolDXLayer
